
from sklearn.experimental import enable_iterative_imputer
from sklearn.impute import IterativeImputer
import numpy as np


def impute(data, imputerType='iterative', ignore_first_index_column=False):

	if type(data).__name__ == 'ndarray': 
		X = data
	elif type(data).__name__ == 'wData': 
		X = data.df.values
	elif type(data).__name__ == 'DataFrame': 
		X = data.values
	else:
		raise

	if ignore_first_index_column:
		c1 = np.atleast_2d(X[:,0]).T
		X = X[:,1:]

	imp_mean = IterativeImputer(random_state=0)
	imp_mean.fit(X)
	Xi = imp_mean.transform(X)

	if ignore_first_index_column:
		Xi = np.hstack((c1,Xi))

	if type(data).__name__ == 'ndarray': 
		return Xi
	elif type(data).__name__ == 'wData': 
		data.df[:] = Xi
		return data
	elif type(data).__name__ == 'DataFrame': 
		data[:] = Xi
		return data





import numpy as np
import torch
from torch.autograd import Variable

def ensure_numpy(data, rounding=None):
	if type(data).__name__ == 'ndarray': 
		if len(data.shape) == 1:
			X = np.atleast_2d(data).T
		else:
			X = data
	elif type(data).__name__ == 'wData': 
		X = data.df.values
	elif type(data).__name__ == 'DataFrame': 
		X = data.values
	elif type(data).__name__ == 'Tensor': 
		X = data.detach().cpu().numpy()
	elif np.isscalar(data):
		X = np.array([[data]])

	print(rounding)
	if rounding is not None: X = np.round(X, rounding)
	return X


def ensure_tensor(data, dataType=torch.FloatTensor):
	if torch.cuda.is_available(): 
		device = 'cuda'
	else: self.device = 'cpu'

	if type(data).__name__ == 'ndarray': 
		x = torch.from_numpy(data)
		x = Variable(x.type(dataType), requires_grad=False)
		X = x.to(device, non_blocking=True )
	elif type(data).__name__ == 'wData': 
		X = data.get_data_as('Tensor')
	elif type(data).__name__ == 'DataFrame': 
		X = data.values
	elif np.isscalar(data):
		X = np.array([[data]])
	elif type(data).__name__ == 'Tensor': 
		x = Variable(data.type(dataType), requires_grad=False)
		X = x.to(device, non_blocking=True )
	else:
		raise ValueError('Unknown dataType %s'%type(data).__name__)

	return X





